@@ -27,28 +27,18 @@ System (version for POSIX threads).
 
    */
 
-/* FIXME bugzilla.eazel.com 1132:
-   check that all the data is freed properly //in the callback dispatching
-   functions//.  */
-
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
-#ifdef HAVE_ALLOCA_H
-#include <alloca.h>
-#endif
+#include "gnome-vfs-job.h"
 
-#include <unistd.h>
 #include <errno.h>
 #include <fcntl.h>
-#include <unistd.h>
 #include <string.h>
+#include <unistd.h>
 
-#include "gnome-vfs.h"
-#include "gnome-vfs-private.h"
-
-#include "gnome-vfs-job.h"
+#include "gnome-vfs-job-slave.h"
 
 #if GNOME_VFS_JOB_DEBUG
 
@@ -65,40 +55,46 @@ static void gnome_vfs_job_release_current_op (GnomeVFSJob *job);
 static void gnome_vfs_job_release_notify_op  (GnomeVFSJob *job);
 static void gnome_vfs_job_finish_destroy     (GnomeVFSJob *job);
 
+static int job_count = 0;
+
 static void
-set_fl(int fd, int flags)
+set_fl (int fd, int flags)
 {
 	int val;
 
-	if ( (val = fcntl(fd, F_GETFL, 0)) < 0 ) {
-		g_warning("fcntl() F_GETFL failed: %s", strerror(errno));
+	val = fcntl (fd, F_GETFL, 0);
+	if (val < 0) {
+		g_warning ("fcntl() F_GETFL failed: %s", strerror (errno));
 		return;
 	}
 
 	val |= flags;
 	
-	if ( (val = fcntl(fd, F_SETFL, val)) < 0 ) {
-		g_warning("fcntl() F_SETFL failed: %s", strerror(errno));
+	val = fcntl (fd, F_SETFL, val);
+	if (val < 0) {
+		g_warning ("fcntl() F_SETFL failed: %s", strerror (errno));
 		return;
-	}		
+	}
 }
 
 static void
-clr_fl(int fd, int flags)
+clr_fl (int fd, int flags)
 {
 	int val;
 
-	if ( (val = fcntl(fd, F_GETFL, 0)) < 0 ) {
-		g_warning("fcntl() F_GETFL failed: %s", strerror(errno));
+	val = fcntl (fd, F_GETFL, 0);
+	if (val < 0) {
+		g_warning ("fcntl() F_GETFL failed: %s", strerror (errno));
 		return;
 	}
 
 	val &= ~flags;
 	
-	if ( (val = fcntl(fd, F_SETFL, val)) < 0 ) {
-		g_warning("fcntl() F_SETFL failed: %s", strerror(errno));
+	val = fcntl (fd, F_SETFL, val);
+	if (val < 0) {
+		g_warning ("fcntl() F_SETFL failed: %s", strerror (errno));
 		return;
-	}		
+	}
 }
 
 
@@ -167,11 +163,13 @@ wakeup (GnomeVFSJob *job)
 	return retval;
 }
 
+
 /* This notifies the master thread asynchronously, without waiting for an
    acknowledgment.  */
 static gboolean
 job_oneway_notify (GnomeVFSJob *job)
 {
+
 	JOB_DEBUG (("lock channel %p", job));
 	g_mutex_lock (job->wakeup_channel_lock);
 
@@ -185,6 +183,7 @@ job_oneway_notify (GnomeVFSJob *job)
 	return wakeup (job);
 }
 
+
 /* This notifies the master threads, waiting until it acknowledges the
    notification.  */
 static gboolean
@@ -258,7 +257,6 @@ job_notify_and_close (GnomeVFSJob *job)
 	return retval;
 }
 
-
 static void
 dispatch_open_callback (GnomeVFSJob *job, GnomeVFSOp *op)
 {
@@ -267,8 +265,6 @@ dispatch_open_callback (GnomeVFSJob *job, GnomeVFSOp *op)
 
 	open_op = &op->specifics.open;
 
-	gnome_vfs_uri_unref (open_op->request.uri);
-
 	callback = (GnomeVFSAsyncOpenCallback) op->callback;
 	(* callback) ((GnomeVFSAsyncHandle *) job,
 		      open_op->notify.result,
@@ -283,8 +279,6 @@ dispatch_create_callback (GnomeVFSJob *job, GnomeVFSOp *op)
 
 	create_op = &op->specifics.create;
 
-	gnome_vfs_uri_unref (create_op->request.uri);
-
 	callback = (GnomeVFSAsyncCreateCallback) op->callback;
 	(* callback) ((GnomeVFSAsyncHandle *) job,
 		      create_op->notify.result,
@@ -298,7 +292,6 @@ dispatch_open_as_channel_callback (GnomeVFSJob *job, GnomeVFSOp *op)
 	GnomeVFSOpenAsChannelOp *open_as_channel_op;
 
 	open_as_channel_op = &op->specifics.open_as_channel;
-	gnome_vfs_uri_unref (open_as_channel_op->request.uri);
 
 	callback = (GnomeVFSAsyncOpenAsChannelCallback) op->callback;
 	(* callback) ((GnomeVFSAsyncHandle *) job,
@@ -315,8 +308,6 @@ dispatch_create_as_channel_callback (GnomeVFSJob *job, GnomeVFSOp *op)
 
 	create_as_channel_op = &op->specifics.create_as_channel;
 
-	gnome_vfs_uri_unref (create_as_channel_op->request.uri);
-
 	callback = (GnomeVFSAsyncCreateAsChannelCallback) op->callback;
 	(* callback) ((GnomeVFSAsyncHandle *) job,
 		      create_as_channel_op->notify.channel,
@@ -382,9 +373,6 @@ dispatch_load_directory_callback (GnomeVFSJob *job, GnomeVFSOp *op)
 
 	load_directory_op = &op->specifics.load_directory;
 
-	if (load_directory_op->notify.result != GNOME_VFS_OK)
-		gnome_vfs_uri_unref (load_directory_op->request.uri);
-
 	callback = (GnomeVFSAsyncDirectoryLoadCallback) op->callback;
 	(* callback) ((GnomeVFSAsyncHandle *) job,
 		      load_directory_op->notify.result,
@@ -397,8 +385,7 @@ static void
 dispatch_get_file_info_callback (GnomeVFSJob *job, GnomeVFSOp *op)
 {
 	GnomeVFSAsyncGetFileInfoCallback callback;
-	GList *result_list, *p;
-	GnomeVFSGetFileInfoResult *result_item;
+	GList *result_list;
 
 	callback = (GnomeVFSAsyncGetFileInfoCallback) op->callback;
 	result_list = op->specifics.get_file_info.notify.result_list;
@@ -406,12 +393,59 @@ dispatch_get_file_info_callback (GnomeVFSJob *job, GnomeVFSOp *op)
 	(* callback) ((GnomeVFSAsyncHandle *) job,
 		      result_list,
 		      op->callback_data);
+}
+
+static void
+free_get_file_info_data (GnomeVFSOp *op)
+{
+	GList *result_list, *p;
+	GnomeVFSGetFileInfoResult *result_item;
+
+	gnome_vfs_uri_list_free (op->specifics.get_file_info.request.uris);
+
+	result_list = op->specifics.get_file_info.notify.result_list;
 
 	for (p = result_list; p != NULL; p = p->next) {
 		result_item = p->data;
 
 		gnome_vfs_uri_unref (result_item->uri);
 		gnome_vfs_file_info_unref (result_item->file_info);
+		g_free (result_item);
+	}
+	g_list_free (result_list);
+}
+
+static void
+dispatch_find_directory_callback (GnomeVFSJob *job, GnomeVFSOp *op)
+{
+	GnomeVFSAsyncFindDirectoryCallback callback;
+	GList *result_list;
+
+	callback = (GnomeVFSAsyncFindDirectoryCallback) op->callback;
+	result_list = op->specifics.find_directory.notify.result_list;
+
+	(* callback) ((GnomeVFSAsyncHandle *) job,
+		      result_list,
+		      op->callback_data);
+}
+
+static void
+free_find_directory_data (GnomeVFSOp *op)
+{
+	GList *result_list, *p;
+	GnomeVFSFindDirectoryResult *result_item;
+
+	gnome_vfs_uri_list_free (op->specifics.find_directory.request.uris);
+
+	result_list = op->specifics.find_directory.notify.result_list;
+
+	for (p = result_list; p != NULL; p = p->next) {
+		result_item = p->data;
+
+		if (result_item->uri != NULL) {
+			gnome_vfs_uri_unref (result_item->uri);
+		}
+		g_free (result_item);
 	}
 	g_list_free (result_list);
 }
@@ -421,22 +455,19 @@ dispatch_set_file_info_callback (GnomeVFSJob *job, GnomeVFSOp *op)
 {
 	GnomeVFSAsyncSetFileInfoCallback callback;
 	gboolean new_info_is_valid;
-
+	
 	new_info_is_valid = 
-		op->specifics.set_file_info.notify.set_file_info_result == GNOME_VFS_OK &&
-		op->specifics.set_file_info.notify.get_file_info_result == GNOME_VFS_OK;
-
+		op->specifics.set_file_info.notify.set_file_info_result == GNOME_VFS_OK
+		&& op->specifics.set_file_info.notify.get_file_info_result == GNOME_VFS_OK;
+	
 	callback = (GnomeVFSAsyncSetFileInfoCallback) op->callback;
-
+	
 	(* callback) ((GnomeVFSAsyncHandle *) job,
 		      op->specifics.set_file_info.notify.set_file_info_result,
 		      new_info_is_valid 
-		      	? &op->specifics.set_file_info.notify.info 
-		      	: NULL,
+		      ? &op->specifics.set_file_info.notify.info 
+		      : NULL,
 		      op->callback_data);
-
-	/* Always clear new info, whether or not it's valid. */
-	gnome_vfs_file_info_clear (&op->specifics.set_file_info.notify.info);
 }
 
 static void
@@ -506,8 +537,17 @@ dispatch_job_callback (GIOChannel *source,
 	
 	JOB_DEBUG (("dispatching %p", job));
 	/* Do the callback, but not if this operation has been cancelled. */
+
 	if (gnome_vfs_context_check_cancellation (op->context)) {
 		switch (op->type) {
+		case GNOME_VFS_OP_CREATE:
+			if (op->specifics.create.notify.result == GNOME_VFS_OK)
+				handle_cancelled_open (job, op);
+			break;
+		case GNOME_VFS_OP_CREATE_AS_CHANNEL:
+			if (op->specifics.create_as_channel.notify.result == GNOME_VFS_OK)
+				handle_cancelled_open (job, op);
+			break;
 		case GNOME_VFS_OP_OPEN:
 			if (op->specifics.open.notify.result == GNOME_VFS_OK)
 				handle_cancelled_open (job, op);
@@ -516,67 +556,58 @@ dispatch_job_callback (GIOChannel *source,
 			if (op->specifics.open_as_channel.notify.result == GNOME_VFS_OK)
 				handle_cancelled_open (job, op);
 			break;
-		case GNOME_VFS_OP_CREATE:
-			if (op->specifics.create.notify.result == GNOME_VFS_OK)
-				handle_cancelled_open (job, op);
-			break;
-		case GNOME_VFS_OP_CREATE_SYMBOLIC_LINK:
-			if (op->specifics.create_symbolic_link.notify.result == GNOME_VFS_OK)
-				handle_cancelled_open (job, op);
-			break;
-		case GNOME_VFS_OP_CREATE_AS_CHANNEL:
-			if (op->specifics.create_as_channel.notify.result == GNOME_VFS_OK)
-				handle_cancelled_open (job, op);
-			break;
 		case GNOME_VFS_OP_CLOSE:
-		case GNOME_VFS_OP_READ:
-		case GNOME_VFS_OP_WRITE:
-		case GNOME_VFS_OP_LOAD_DIRECTORY:
-		case GNOME_VFS_OP_XFER:
+		case GNOME_VFS_OP_CREATE_SYMBOLIC_LINK:
+		case GNOME_VFS_OP_FIND_DIRECTORY:
 		case GNOME_VFS_OP_GET_FILE_INFO:
+		case GNOME_VFS_OP_LOAD_DIRECTORY:
+		case GNOME_VFS_OP_READ:
 		case GNOME_VFS_OP_SET_FILE_INFO:
+		case GNOME_VFS_OP_WRITE:
+		case GNOME_VFS_OP_XFER:
 			break;
 		}
 	} else {
 		switch (op->type) {
+		case GNOME_VFS_OP_CLOSE:
+			dispatch_close_callback (job, op);
+			break;
+		case GNOME_VFS_OP_CREATE:
+			dispatch_create_callback (job, op);
+			break;
+		case GNOME_VFS_OP_CREATE_AS_CHANNEL:
+			dispatch_create_as_channel_callback (job, op);
+			break;
+		case GNOME_VFS_OP_CREATE_SYMBOLIC_LINK:
+			dispatch_create_callback (job, op);
+			break;
+		case GNOME_VFS_OP_FIND_DIRECTORY:
+			dispatch_find_directory_callback (job, op);
+			break;
+		case GNOME_VFS_OP_GET_FILE_INFO:
+			dispatch_get_file_info_callback (job, op);
+			break;
+		case GNOME_VFS_OP_LOAD_DIRECTORY:
+			dispatch_load_directory_callback (job, op);
+			break;
 		case GNOME_VFS_OP_OPEN:
 			dispatch_open_callback (job, op);
 			break;
 		case GNOME_VFS_OP_OPEN_AS_CHANNEL:
 			dispatch_open_as_channel_callback (job, op);
 			break;
-		case GNOME_VFS_OP_CREATE:
-			dispatch_create_callback (job, op);
-			break;
-		case GNOME_VFS_OP_CREATE_SYMBOLIC_LINK:
-			dispatch_create_callback (job, op);
-			break;
-		case GNOME_VFS_OP_CREATE_AS_CHANNEL:
-			dispatch_create_as_channel_callback (job, op);
-			break;
-		case GNOME_VFS_OP_CLOSE:
-			dispatch_close_callback (job, op);
-			break;
 		case GNOME_VFS_OP_READ:
 			dispatch_read_callback (job, op);
 			break;
-		case GNOME_VFS_OP_WRITE:
-			dispatch_write_callback (job, op);
-			break;
-		case GNOME_VFS_OP_LOAD_DIRECTORY:
-			dispatch_load_directory_callback (job, op);
-			break;
-		case GNOME_VFS_OP_XFER:
-			dispatch_xfer_callback (job, op);
-			break;
-		case GNOME_VFS_OP_GET_FILE_INFO:
-			dispatch_get_file_info_callback (job, op);
-			break;
 		case GNOME_VFS_OP_SET_FILE_INFO:
 			dispatch_set_file_info_callback (job, op);
 			break;
-		default:
-			g_warning (_("Unknown job ID %d"), op->type);
+		case GNOME_VFS_OP_WRITE:
+			dispatch_write_callback (job, op);
+			break;
+		case GNOME_VFS_OP_XFER:
+			dispatch_xfer_callback (job, op);
+			break;
 		}
 	}
 
@@ -586,64 +617,44 @@ dispatch_job_callback (GIOChannel *source,
 	return TRUE;
 }
 
-
 GnomeVFSJob *
 gnome_vfs_job_new (void)
 {
-	GnomeVFSJobSlave *slave;
 	GnomeVFSJob *new_job;
 	gint pipefd[2];
-	gchar c;
-	guint bytes_read;
-	GIOError error;
-
+	
 	if (pipe (pipefd) != 0) {
 		g_warning ("Cannot create pipe for the new GnomeVFSJob: %s",
 			   g_strerror (errno));
 		return NULL;
 	}
-
+	
 	new_job = g_new0 (GnomeVFSJob, 1);
-
+	
 	new_job->access_lock = g_mutex_new ();
 	new_job->execution_condition = g_cond_new ();
 	new_job->notify_ack_condition = g_cond_new ();
 	new_job->notify_ack_lock = g_mutex_new ();
-
+	
 	new_job->is_empty = TRUE;
-
+	
 	new_job->wakeup_channel_in = g_io_channel_unix_new (pipefd[0]);
 	new_job->wakeup_channel_out = g_io_channel_unix_new (pipefd[1]);
 	new_job->wakeup_channel_lock = g_mutex_new ();
-
-	g_io_add_watch_full (new_job->wakeup_channel_in, G_PRIORITY_LOW, G_IO_IN,
+	
+	g_io_add_watch_full (new_job->wakeup_channel_in, G_PRIORITY_HIGH, G_IO_IN,
 			     dispatch_job_callback, new_job, NULL);
-
-	slave = gnome_vfs_job_slave_new (new_job);
-	if (slave == NULL) {
+	
+	if (!gnome_vfs_job_create_slave (new_job)) {
 		g_warning ("Cannot create job slave.");
+		/* FIXME bugzilla.eazel.com 3833: A lot of leaked objects here. */
 		g_free (new_job);
 		return NULL;
 	}
-
-	new_job->slave = slave;
-
+	
 	JOB_DEBUG (("new job %p", new_job));
 
-	/* Wait for the thread to come up. 
-	 *
-	 * Keep reading until we get the synch character from the slave thread --
-	 * When the thread receives a signal, the read will return with 
-	 * nothing the first time but will succeed if we retry.
-	 */
-	for (;;) {
-		error = g_io_channel_read (new_job->wakeup_channel_in, &c, 1, &bytes_read);
-		JOB_DEBUG (("new job ready to rip %p, %c, %d, %d", 
-			new_job, c, bytes_read, error));
-		JOB_DEBUG_ONLY(if (error !=  G_IO_ERROR_NONE) perror("system error:"));
-		if (bytes_read > 0)
-			break;
-	}
+	job_count++;
 
 	return new_job;
 }
@@ -683,12 +694,74 @@ gnome_vfs_job_finish_destroy (GnomeVFSJob *job)
 	JOB_DEBUG (("job %p terminated cleanly", job));
 
 	g_free (job);
+
+	job_count--;
+}
+
+int
+gnome_vfs_job_get_count (void)
+{
+	return job_count;
 }
 
-
 static void
 gnome_vfs_op_destroy (GnomeVFSOp *op)
 {
+	switch (op->type) {
+	case GNOME_VFS_OP_CREATE:
+		if (op->specifics.create.request.uri != NULL) {
+			gnome_vfs_uri_unref (op->specifics.create.request.uri);
+		}
+		break;
+	case GNOME_VFS_OP_CREATE_AS_CHANNEL:
+		if (op->specifics.create_as_channel.request.uri != NULL) {
+			gnome_vfs_uri_unref (op->specifics.create_as_channel.request.uri);
+		}
+		break;
+	case GNOME_VFS_OP_CREATE_SYMBOLIC_LINK:
+		gnome_vfs_uri_unref (op->specifics.create_symbolic_link.request.uri);
+		g_free (op->specifics.create_symbolic_link.request.uri_reference);
+		break;
+	case GNOME_VFS_OP_FIND_DIRECTORY:
+		free_find_directory_data (op);
+		break;
+	case GNOME_VFS_OP_GET_FILE_INFO:
+		free_get_file_info_data (op);
+		break;
+	case GNOME_VFS_OP_LOAD_DIRECTORY:
+		if (op->specifics.load_directory.request.uri != NULL) {
+			gnome_vfs_uri_unref (op->specifics.load_directory.request.uri);
+		}
+		g_free (op->specifics.load_directory.request.sort_rules);
+		g_free (op->specifics.load_directory.request.filter_pattern);
+		break;
+	case GNOME_VFS_OP_OPEN:
+		if (op->specifics.open.request.uri != NULL) {
+			gnome_vfs_uri_unref (op->specifics.open.request.uri);
+		}
+		break;
+	case GNOME_VFS_OP_OPEN_AS_CHANNEL:
+		if (op->specifics.open_as_channel.request.uri != NULL) {
+			gnome_vfs_uri_unref (op->specifics.open_as_channel.request.uri);
+		}
+		break;
+	case GNOME_VFS_OP_SET_FILE_INFO:
+		gnome_vfs_uri_unref (op->specifics.set_file_info.request.uri);
+		gnome_vfs_file_info_clear (&op->specifics.set_file_info.request.info);
+		gnome_vfs_file_info_clear (&op->specifics.set_file_info.notify.info);
+		break;
+	case GNOME_VFS_OP_XFER:
+		gnome_vfs_uri_list_free (op->specifics.xfer.request.source_uri_list);
+		gnome_vfs_uri_list_free (op->specifics.xfer.request.target_uri_list);
+		break;
+	case GNOME_VFS_OP_CLOSE:
+	case GNOME_VFS_OP_READ:
+	case GNOME_VFS_OP_WRITE:
+		break;
+	default:
+		g_warning (_("Unknown job ID %d"), op->type);
+	}
+	
 	gnome_vfs_context_unref (op->context);
 	g_free (op);
 }
@@ -825,9 +898,9 @@ serve_channel_read (GnomeVFSHandle *handle,
 				if (bytes_read == 0) {
 					int fd;
 
-					fd = g_io_channel_unix_get_fd(channel_out);
+					fd = g_io_channel_unix_get_fd (channel_out);
 					
-					clr_fl(fd, O_NONBLOCK);
+					clr_fl (fd, O_NONBLOCK);
 				} else {
 					if (written_bytes_in_buffer > 0) {
 						/* Need to shift the unwritten bytes
@@ -868,7 +941,7 @@ serve_channel_read (GnomeVFSHandle *handle,
 	}
 
  end:
-	g_free(buffer);
+	g_free (buffer);
 	g_io_channel_close (channel_out);
 	g_io_channel_unref (channel_out);
 	g_io_channel_unref (channel_in);
@@ -880,11 +953,10 @@ serve_channel_write (GnomeVFSHandle *handle,
 		     GIOChannel *channel_out,
 		     GnomeVFSContext *context)
 {
-	gpointer buffer;
+	gchar buffer[DEFAULT_BUFFER_SIZE];
 	guint buffer_size;
 
 	buffer_size = DEFAULT_BUFFER_SIZE;
-	buffer = alloca (buffer_size);
 
 	while (1) {
 		GnomeVFSResult result;
@@ -926,7 +998,6 @@ serve_channel_write (GnomeVFSHandle *handle,
 	g_io_channel_unref (channel_out);
 }
 
-
 /* Job execution.  This is performed by the slave thread.  */
 
 static gboolean
@@ -939,11 +1010,14 @@ execute_open (GnomeVFSJob *job)
 
 	open_op = &job->current_op->specifics.open;
 
-	result = gnome_vfs_open_uri_cancellable (&handle, open_op->request.uri,
-						 open_op->request.open_mode,
-						 job->current_op->context);
-
-	job->handle = handle;
+	if (open_op->request.uri == NULL) {
+		result = GNOME_VFS_ERROR_INVALID_URI;
+	} else {
+		result = gnome_vfs_open_uri_cancellable (&handle, open_op->request.uri,
+							 open_op->request.open_mode,
+							 job->current_op->context);
+		job->handle = handle;
+	}
 	open_op->notify.result = result;
 
 	notify_retval = job_oneway_notify_and_close (job);
@@ -966,11 +1040,15 @@ execute_open_as_channel (GnomeVFSJob *job)
 
 	open_as_channel_op = &job->current_op->specifics.open_as_channel;
 
-	result = gnome_vfs_open_uri_cancellable
-		(&handle,
-		 open_as_channel_op->request.uri,
-		 open_as_channel_op->request.open_mode,
-		 job->current_op->context);
+	if (open_as_channel_op->request.uri == NULL) {
+		result = GNOME_VFS_ERROR_INVALID_URI;
+	} else {
+		result = gnome_vfs_open_uri_cancellable
+			(&handle,
+			 open_as_channel_op->request.uri,
+			 open_as_channel_op->request.open_mode,
+			 job->current_op->context);
+	}
 
 	if (result != GNOME_VFS_OK) {
 		open_as_channel_op->notify.channel = NULL;
@@ -989,9 +1067,10 @@ execute_open_as_channel (GnomeVFSJob *job)
 	}
 
 	/* Set up the pipe for nonblocking writes, so if the main
-           thread is blocking for some reason the slave can keep
-           reading data. */
-	set_fl(pipefd[1], O_NONBLOCK);
+	 * thread is blocking for some reason the slave can keep
+	 * reading data.
+	 */
+	set_fl (pipefd[1], O_NONBLOCK);
 	
 	channel_in = g_io_channel_unix_new (pipefd[0]);
 	channel_out = g_io_channel_unix_new (pipefd[1]);
@@ -1031,15 +1110,19 @@ execute_create (GnomeVFSJob *job)
 
 	create_op = &job->current_op->specifics.create;
 
-	result = gnome_vfs_create_uri_cancellable
-						(&handle,
-						 create_op->request.uri,
-						 create_op->request.open_mode,
-						 create_op->request.exclusive,
-						 create_op->request.perm,
-						 job->current_op->context);
-
-	job->handle = handle;
+	if (create_op->request.uri == NULL) {
+		result = GNOME_VFS_ERROR_INVALID_URI;
+	} else {
+		result = gnome_vfs_create_uri_cancellable
+			(&handle,
+			 create_op->request.uri,
+			 create_op->request.open_mode,
+			 create_op->request.exclusive,
+			 create_op->request.perm,
+			 job->current_op->context);
+		
+		job->handle = handle;
+	}
 	create_op->notify.result = result;
 
 	notify_retval = job_oneway_notify_and_close (job);
@@ -1054,18 +1137,16 @@ static gboolean
 execute_create_symbolic_link (GnomeVFSJob *job)
 {
 	GnomeVFSResult result;
-	GnomeVFSHandle *handle;
 	GnomeVFSCreateLinkOp *create_op;
 	gboolean notify_retval;
 
 	create_op = &job->current_op->specifics.create_symbolic_link;
 
 	result = gnome_vfs_create_symbolic_link_cancellable
-						(create_op->request.uri,
-						 create_op->request.uri_reference,
-						 job->current_op->context);
+		(create_op->request.uri,
+		 create_op->request.uri_reference,
+		 job->current_op->context);
 
-	job->handle = handle;
 	create_op->notify.result = result;
 
 	notify_retval = job_oneway_notify_and_close (job);
@@ -1087,12 +1168,16 @@ execute_create_as_channel (GnomeVFSJob *job)
 
 	create_as_channel_op = &job->current_op->specifics.create_as_channel;
 
-	result = gnome_vfs_open_uri_cancellable
-		(&handle,
-		 create_as_channel_op->request.uri,
-		 create_as_channel_op->request.open_mode,
-		 job->current_op->context);
-
+	if (create_as_channel_op->request.uri == NULL) {
+		result = GNOME_VFS_ERROR_INVALID_URI;
+	} else {
+		result = gnome_vfs_open_uri_cancellable
+			(&handle,
+			 create_as_channel_op->request.uri,
+			 create_as_channel_op->request.open_mode,
+			 job->current_op->context);
+	}
+	
 	if (result != GNOME_VFS_OK) {
 		create_as_channel_op->notify.channel = NULL;
 		create_as_channel_op->notify.result = result;
@@ -1190,11 +1275,15 @@ execute_load_directory_not_sorted (GnomeVFSJob *job,
 	JOB_DEBUG (("%p", job));
 	load_directory_op = &job->current_op->specifics.load_directory;
 
-	result = gnome_vfs_directory_open_from_uri
-		(&handle,
-		 load_directory_op->request.uri,
-		 load_directory_op->request.options,
-		 filter);
+	if (load_directory_op->request.uri == NULL) {
+		result = GNOME_VFS_ERROR_INVALID_URI;
+	} else {
+		result = gnome_vfs_directory_open_from_uri
+			(&handle,
+			 load_directory_op->request.uri,
+			 load_directory_op->request.options,
+			 filter);
+	}
 
 	if (result != GNOME_VFS_OK) {
 		load_directory_op->notify.result = result;
@@ -1209,14 +1298,14 @@ execute_load_directory_not_sorted (GnomeVFSJob *job,
 
 	count = 0;
 	while (1) {
-		info = gnome_vfs_file_info_new ();
-
 		if (gnome_vfs_context_check_cancellation (job->current_op->context)) {
 			JOB_DEBUG (("cancelled, bailing %p", job));
 			result = GNOME_VFS_ERROR_CANCELLED;
 			break;
 		}
 		
+		info = gnome_vfs_file_info_new ();
+
 		result = gnome_vfs_directory_read_next (handle, info);
 
 		if (result == GNOME_VFS_OK) {
@@ -1277,11 +1366,15 @@ execute_load_directory_sorted (GnomeVFSJob *job,
 	JOB_DEBUG (("%p", job));
 	load_directory_op = &job->current_op->specifics.load_directory;
 
-	result = gnome_vfs_directory_list_load_from_uri
-		(&directory_list,
-		 load_directory_op->request.uri,
-		 load_directory_op->request.options,
-		 filter);
+	if (load_directory_op->request.uri == NULL) {
+		result = GNOME_VFS_ERROR_INVALID_URI;
+	} else {
+		result = gnome_vfs_directory_list_load_from_uri
+			(&directory_list,
+			 load_directory_op->request.uri,
+			 load_directory_op->request.options,
+			 filter);
+	}
 
 	if (result != GNOME_VFS_OK) {
 		load_directory_op->notify.result = result;
@@ -1342,7 +1435,6 @@ execute_get_file_info (GnomeVFSJob *job)
 
 	gijob = &job->current_op->specifics.get_file_info;
 
-	gijob->notify.result_list = NULL;
 	for (p = gijob->request.uris; p != NULL; p = p->next) {
 		result_item = g_new (GnomeVFSGetFileInfoResult, 1);
 
@@ -1368,6 +1460,7 @@ static gboolean
 execute_set_file_info (GnomeVFSJob *job)
 {
 	GnomeVFSSetFileInfoOp *op;
+	GnomeVFSURI *parent_uri, *uri_after;
 
 	op = &job->current_op->specifics.set_file_info;
 
@@ -1375,23 +1468,71 @@ execute_set_file_info (GnomeVFSJob *job)
 		(op->request.uri, &op->request.info, op->request.mask,
 		 job->current_op->context);
 
-	/* FIXME: potential leak if operation is cancelled before getting here. */
-	gnome_vfs_file_info_clear (&op->request.info);
+	/* Get the new URI after the set_file_info. The name may have
+	 * changed.
+	 */
+	uri_after = NULL;
+	if (op->notify.set_file_info_result == GNOME_VFS_OK
+	    && (op->request.mask & GNOME_VFS_SET_FILE_INFO_NAME) != 0) {
+		parent_uri = gnome_vfs_uri_get_parent (op->request.uri);
+		if (parent_uri != NULL) {
+			uri_after = gnome_vfs_uri_append_file_name
+				(parent_uri, op->request.info.name);
+			gnome_vfs_uri_unref (parent_uri);
+		}
+	}
+	if (uri_after == NULL) {
+		uri_after = op->request.uri;
+		gnome_vfs_uri_ref (uri_after);
+	}
 
-	/* Always get new file info, even if setter failed. Init here and clear
-	 * in dispatch_set_file_info.
+	/* Always get new file info, even if setter failed. Init here
+	 * and clear in dispatch_set_file_info.
 	 */
 	gnome_vfs_file_info_init (&op->notify.info);
-	op->notify.get_file_info_result = gnome_vfs_get_file_info_uri_cancellable
-		(op->request.uri,
-		 &op->notify.info,
-		 op->request.options,
-		 job->current_op->context);
+	if (uri_after == NULL) {
+		op->notify.get_file_info_result = GNOME_VFS_ERROR_INVALID_URI;
+	} else {
+		op->notify.get_file_info_result = gnome_vfs_get_file_info_uri_cancellable
+			(uri_after,
+			 &op->notify.info,
+			 op->request.options,
+			 job->current_op->context);
+		gnome_vfs_uri_unref (uri_after);
+	}
 
 	job_oneway_notify_and_close (job);
 	return FALSE;
 }
 
+static gboolean
+execute_find_directory (GnomeVFSJob *job)
+{
+	GnomeVFSFindDirectoryOp *op;
+	GList *p;
+	GnomeVFSGetFileInfoResult *result_item;
+
+	op = &job->current_op->specifics.find_directory;
+	for (p = op->request.uris; p != NULL; p = p->next) {
+		result_item = g_new (GnomeVFSGetFileInfoResult, 1);
+
+		result_item->result = gnome_vfs_find_directory_cancellable
+			((GnomeVFSURI *) p->data,
+			 op->request.kind,
+			 &result_item->uri,
+			 op->request.create_if_needed,
+			 op->request.find_if_needed,
+			 op->request.permissions,
+			 job->current_op->context);
+		op->notify.result_list = g_list_prepend (op->notify.result_list, result_item);
+	}
+
+	op->notify.result_list = g_list_reverse (op->notify.result_list);
+	
+	job_oneway_notify_and_close (job);
+	return FALSE;
+}
+
 static gboolean
 execute_load_directory (GnomeVFSJob *job)
 {
@@ -1418,14 +1559,9 @@ execute_load_directory (GnomeVFSJob *job)
 
 	job_close (job);
 
-	g_free (load_directory_op->request.sort_rules);
-	g_free (load_directory_op->request.filter_pattern);
-
-
 	return FALSE;
 }
 
-
 static gint
 xfer_callback (GnomeVFSXferProgressInfo *info,
 	       gpointer data)
@@ -1453,10 +1589,8 @@ execute_xfer (GnomeVFSJob *job)
 
 	xfer_op = &job->current_op->specifics.xfer;
 
-	result = gnome_vfs_xfer_private (xfer_op->request.source_directory_uri,
-					 xfer_op->request.source_name_list,
-					 xfer_op->request.target_directory_uri,
-					 xfer_op->request.target_name_list,
+	result = gnome_vfs_xfer_private (xfer_op->request.source_uri_list,
+					 xfer_op->request.target_uri_list,
 					 xfer_op->request.xfer_options,
 					 xfer_op->request.error_mode,
 					 xfer_op->request.overwrite_mode,
@@ -1493,7 +1627,6 @@ execute_xfer (GnomeVFSJob *job)
 	return FALSE;
 }
 
-
 /* This function is called by the slave thread to execute a
    GnomeVFSJob.  */
 gboolean
@@ -1527,6 +1660,8 @@ gnome_vfs_job_execute (GnomeVFSJob *job)
 		return execute_write (job);
 	case GNOME_VFS_OP_LOAD_DIRECTORY:
 		return execute_load_directory (job);
+	case GNOME_VFS_OP_FIND_DIRECTORY:
+		return execute_find_directory (job);
 	case GNOME_VFS_OP_XFER:
 		return execute_xfer (job);
 	case GNOME_VFS_OP_GET_FILE_INFO:
@@ -1540,7 +1675,7 @@ gnome_vfs_job_execute (GnomeVFSJob *job)
 }
 
 
-GnomeVFSResult
+void
 gnome_vfs_job_cancel (GnomeVFSJob *job)
 {
 	GnomeVFSOp *op;
@@ -1548,14 +1683,14 @@ gnome_vfs_job_cancel (GnomeVFSJob *job)
 
 	JOB_DEBUG (("async cancel %p", job));
 
-	g_return_val_if_fail (job != NULL, GNOME_VFS_ERROR_BAD_PARAMETERS);
+	g_return_if_fail (job != NULL);
 
 	op = job->current_op;
 	if (op == NULL) {
 		op = job->notify_op;
 	}
 
-	g_return_val_if_fail (op != NULL, GNOME_VFS_ERROR_BAD_PARAMETERS);
+	g_return_if_fail (op != NULL);
 
 	cancellation = gnome_vfs_context_get_cancellation (op->context);
 	if (cancellation != NULL) {
@@ -1573,6 +1708,4 @@ gnome_vfs_job_cancel (GnomeVFSJob *job)
 	 * set the expectations right.
 	 */
 	JOB_DEBUG (("done cancelling %p", job));
-	
-	return GNOME_VFS_OK;
 }
