@@ -26,9 +26,7 @@
 #define GNOME_VFS_MIME_HANDLERS_H
 
 #include <glib.h>
-#ifdef USING_OAF
 #include <liboaf/liboaf.h>
-#endif
 #include <libgnomevfs/gnome-vfs-types.h> /* for GnomeVFSResult */
 
 typedef enum {
@@ -49,11 +47,8 @@ typedef struct {
 typedef struct {
 	GnomeVFSMimeActionType action_type;
 	union {
-#ifdef USING_OAF
 		OAF_ServerInfo *component;
-#else
 		void *dummy_component;
-#endif
 		GnomeVFSMimeApplication *application;
 	} action;
 } GnomeVFSMimeAction;
@@ -63,9 +58,7 @@ GnomeVFSMimeActionType   gnome_vfs_mime_get_default_action_type            (cons
 GnomeVFSMimeAction *     gnome_vfs_mime_get_default_action                 (const char              *mime_type);
 GnomeVFSMimeAction *     gnome_vfs_mime_get_default_action_without_fallback(const char              *mime_type);
 GnomeVFSMimeApplication *gnome_vfs_mime_get_default_application            (const char              *mime_type);
-#ifdef USING_OAF
 OAF_ServerInfo *         gnome_vfs_mime_get_default_component              (const char              *mime_type);
-#endif
 GList *                  gnome_vfs_mime_get_short_list_applications        (const char              *mime_type);
 GList *                  gnome_vfs_mime_get_short_list_components          (const char              *mime_type);
 GList *                  gnome_vfs_mime_get_all_applications               (const char              *mime_type);
@@ -76,9 +69,10 @@ GnomeVFSResult           gnome_vfs_mime_set_default_application            (cons
 									    const char              *application_id);
 GnomeVFSResult           gnome_vfs_mime_set_default_component              (const char              *mime_type,
 									    const char              *component_iid);
-const char  		 *gnome_vfs_mime_get_icon 			   (const char 		    *mime_type);
+const char  		*gnome_vfs_mime_get_icon 			   (const char 		    *mime_type);
 GnomeVFSResult  	 gnome_vfs_mime_set_icon 			   (const char 		    *mime_type,
 									    const char		    *filename);
+const char 	        *gnome_vfs_mime_get_description   		   (const char              *mime_type);
 
 /* Stored as delta to current user level - API function computes delta and stores in prefs */
 GnomeVFSResult           gnome_vfs_mime_set_short_list_applications        (const char              *mime_type,
@@ -106,8 +100,6 @@ GnomeVFSResult           gnome_vfs_mime_extend_all_applications            (cons
 GnomeVFSResult           gnome_vfs_mime_remove_from_all_applications       (const char              *mime_type,
 									    GList                   *application_ids);
 GnomeVFSMimeApplication *gnome_vfs_mime_application_new_from_id            (const char              *id);
-GnomeVFSResult           gnome_vfs_mime_define_application                 (const char 		    *mime_type,
-									    GnomeVFSMimeApplication *application);
 void                     gnome_vfs_mime_application_free                   (GnomeVFSMimeApplication *application);
 void                     gnome_vfs_mime_action_free                        (GnomeVFSMimeAction      *action);
 
