@@ -95,10 +95,10 @@ static gboolean       do_is_local       (GnomeVFSMethod *method,
 
 static GnomeVFSMethod method = {
         do_open,
-        NULL, /* do_create */
+        NULL, /* create */
         do_close,
-        NULL, /* do_read */
-        NULL, /* do_write */
+        NULL, /* read */
+        NULL, /* write */
         NULL, /* seek */
         NULL, /* tell */
         NULL, /* truncate */
@@ -106,12 +106,16 @@ static GnomeVFSMethod method = {
 	do_close_directory,
         do_read_directory,
         do_get_file_info,
-	NULL, /* do_get_file_info_from_handle */
+	NULL, /* get_file_info_from_handle */
         do_is_local,
-        NULL, /* rename */
 	NULL, /* make directory */
         NULL, /* remove directory */
-	NULL /* unlink */
+	NULL, /* unlink */
+	NULL, /* check_same_fs */
+	NULL, /* set_file_info */
+	NULL, /* truncate */
+	NULL, /* find_directory */
+	NULL /* create_symbolic_link */
 };
 
 static GConfClient *client = NULL;
@@ -244,7 +248,7 @@ set_mime_type_dir (GnomeVFSFileInfo *info,
                    const gchar *dirname,
                    GnomeVFSFileInfoOptions options)
 {
-        info->mime_type = g_strdup ("x-special/directory");
+        info->mime_type = g_strdup ("x-directory/normal");
 
 	return GNOME_VFS_OK;
 }
@@ -482,7 +486,8 @@ do_close_directory (GnomeVFSMethod *method,
         return GNOME_VFS_OK;
 }
 
-/* FIXME: I dunno. There must be something to d there with metedata
+/* FIXME bugzilla.eazel.com 2794: 
+   I dunno. There must be something to d there with metedata
    -- Mathieu */
 static GnomeVFSResult
 file_info_value (GnomeVFSFileInfo *info,
@@ -557,7 +562,7 @@ read_directory (DirectoryHandle *handle,
 		GConfEntry *pair = handle->pairs->data;
                 result = file_info_value (file_info, handle->options,
                                           pair->value, pair->key);
-                gconf_entry_destroy (handle->pairs->data);
+                gconf_entry_free (handle->pairs->data);
                 tmp = g_slist_next (handle->subdirs);
                 
 		g_slist_free_1 (handle->pairs);
