@@ -63,7 +63,7 @@
 
 #include "http-method.h"
 
-#if 0
+#if 1
 #include <stdio.h>
 #include <stdarg.h>
 #include <pthread.h>
@@ -856,6 +856,9 @@ http_status_to_vfs_result (guint status)
 	 */
 
 	switch (status) {
+	case HTTP_STATUS_PRECONDITION_FAILED:
+		/* This mapping is certainly true for MOVE with Overwrite: F, otherwise not so true */
+		return GNOME_VFS_ERROR_FILE_EXISTS;
 	case HTTP_STATUS_UNAUTHORIZED:
 	case HTTP_STATUS_PROXY_AUTH_REQUIRED:
 	case HTTP_STATUS_FORBIDDEN:
@@ -2272,11 +2275,11 @@ do_get_file_info (GnomeVFSMethod *method,
 		} else {
 			g_assert (handle == NULL); /* Make sure we're not leaking some old one */
 
-				/* Lame buggy servers (eg: www.mozilla.org, www.corel.com)) return
-				 * an HTTP error for a HEAD where a GET would succeed. In these cases
-				 * lets try to do a GET.
-				 */
-			
+			/* Lame buggy servers (eg: www.mozilla.org,
+			 * www.corel.com)) return an HTTP error for a
+			 * HEAD where a GET would succeed. In these
+			 * cases lets try to do a GET.
+			 */
 			if (result != GNOME_VFS_OK) {
 				g_assert (handle == NULL); /* Make sure we're not leaking some old one */
 				result = make_request (&handle, uri, "GET", NULL, NULL, context);
@@ -2284,7 +2287,18 @@ do_get_file_info (GnomeVFSMethod *method,
 					gnome_vfs_file_info_copy (file_info, handle->file_info);
 					http_handle_close (handle, context);
 				}
-				/* evil hack */
+
+				/* If we get a redirect, we should be
+				 * basing the MIME type on the type of
+				 * the page we'll be redirected
+				 * too. Maybe we even want to take the
+				 * "follow_links" setting into account.
+				 */
+				/* FIXME: For now we treat all
+				 * redirects as if they lead to a
+				 * text/html. That works pretty well,
+				 * but it's not correct.
+				 */
 				if (handle != NULL && HTTP_REDIRECTED (handle->server_status)) {
 					g_free (file_info->mime_type);
 					file_info->mime_type = g_strdup ("text/html");
@@ -2429,10 +2443,13 @@ do_move (GnomeVFSMethod *method,
 	 gboolean force_replace,
 	 GnomeVFSContext *context)
 {
-	/* FIXME: Ignores force_replace. */
 
-	/* MOVE /path1 HTTP/1.0
-	 * Destination: /path2 */
+	/*
+	 * MOVE /path1 HTTP/1.0
+	 * Destination: /path2
+	 * Overwrite: (T|F)
+	 */
+
 	HttpFileHandle *handle;
 	GnomeVFSResult result;
 
@@ -2445,8 +2462,7 @@ do_move (GnomeVFSMethod *method,
 	}	
 
 	destpath = gnome_vfs_uri_to_string(new_uri, GNOME_VFS_URI_HIDE_USER_NAME|GNOME_VFS_URI_HIDE_PASSWORD);
-	destheader = g_strdup_printf("Destination: %s\r\n", destpath);
-
+	destheader = g_strdup_printf("Destination: %s\r\nOverwrite: %c\r\n", destpath, force_replace ? 'T' : 'F' );
 
 	result = make_request (&handle, old_uri, "MOVE", NULL, destheader, context);
 	http_handle_close (handle, context);
