@@ -501,12 +501,42 @@ do_mv (void)
 	g_free (msg);
 }
 
+static void
+do_findtrash (void)
+{
+	char *from;
+	char *uri_as_string;
+	GnomeVFSResult    result;
+	GnomeVFSURI *from_uri;
+	GnomeVFSURI *result_vfs_uri;
+
+	from = get_fname ();
+
+	from_uri = gnome_vfs_uri_new (from);
+	result = gnome_vfs_find_directory (from_uri, 
+					   GNOME_VFS_DIRECTORY_KIND_TRASH, 
+					   &result_vfs_uri, 
+					   TRUE, TRUE, 0777);
+
+	if (result != GNOME_VFS_OK) {
+		fprintf (stdout, "couldn't find or create trash there, error code %d", result);
+	} else {
+		uri_as_string = gnome_vfs_uri_to_string (result_vfs_uri, GNOME_VFS_URI_HIDE_NONE);
+		fprintf (stdout, "trash found or created here: %s", uri_as_string);
+		g_free (uri_as_string);
+	}
+
+	gnome_vfs_uri_unref (from_uri);
+	gnome_vfs_uri_unref (result_vfs_uri);
+}
+
 static void
 do_info (void)
 {
 	char             *from;
 	GnomeVFSResult    result;
 	GnomeVFSFileInfo *info;
+	const char *mime_type;
 
 	from = get_fname ();
 
@@ -557,11 +587,17 @@ do_info (void)
 	if (info->valid_fields & GNOME_VFS_FILE_INFO_FIELDS_SIZE) {
 		long i = info->size;
 		fprintf (stdout, "Size: %ld bytes", i);
-	} else
+	} else {
 		fprintf (stdout, "Size invalid");
+	}
 	fprintf (stdout, "\n");
 
-	/* FIXME: hack here; should dump them all */
+	mime_type = gnome_vfs_file_info_get_mime_type (info);
+
+	fprintf (stdout, "Mime Type: %s \n", mime_type);
+
+	fprintf (stdout, "\n");
+	/* FIXME bugzilla.eazel.com 2800: hack here; should dump them all */
 	    
 	gnome_vfs_file_info_unref (info);
 }
@@ -844,6 +880,9 @@ main (int argc, char **argv)
 	gnome_init_with_popt_table ("test-vfs", "0.0", argc, argv,
 				    options, 0, &popt_context);
 
+	/* make the stupid "SaveYourself" warning not come up */
+	gnome_client_disconnect (gnome_master_client ());
+
 	if (interactive)
 		vfserr = stderr;
 	else
@@ -873,7 +912,7 @@ main (int argc, char **argv)
 		char *ptr;
 
 		if (interactive) {
-			fprintf (stdout,"%s > ", cur_dir);
+			fprintf (stdout,"\n%s > ", cur_dir);
 			fflush (stdout);
 		}
 		fgets (buffer, 1023, stdin);
@@ -912,6 +951,8 @@ main (int argc, char **argv)
 		else if (g_strcasecmp (ptr, "info") == 0 ||
 			 g_strcasecmp (ptr, "stat") == 0)
 			do_info ();
+		else if (g_strcasecmp (ptr, "findtrash") == 0)
+			do_findtrash ();
 		else if (g_strcasecmp (ptr, "sync") == 0)
 			fprintf (vfserr, "a shell is like a boat, it lists or syncs (RMS)\n");
 		else if (g_strcasecmp (ptr,"help") == 0 ||
